

% vertical levels

kmax = 15; 

% horizontal levels
imax = 100;


x = linspace(-50e3,50e3,imax)';
b = 1000 + -800*exp(-(x/20e3).^2);

dx = x(2)-x(1);

sigma = linspace(-1,0,kmax+1);

% vertical levels
Z_w = b * sigma;
X_w = repmat(x,[1 kmax+1]);

Z_r = (Z_w(:,1:end-1) + Z_w(:,2:end))/2;
X_r = repmat(x,[1 kmax]);

% variables at u
Z_u = (Z_r(1:end-1,:) + Z_r(2:end,:))/2;
X_u = (X_r(1:end-1,:) + X_r(2:end,:))/2;

dZ = Z_w(:,2:end) - Z_w(:,1:end-1);

% density
rho = 1020 + 30 * exp(Z_r/600);

g = 9.81;

% pressure

k = kmax:-1:1;
p_w = zeros(imax,kmax);
p_w(:,kmax+1) = 0; % amtospheric pressure
p_w(:,k) = g * cumsum(rho(:,k) .* dZ(:,k),2);

p_r = (p_w(:,1:end-1) + p_w(:,2:end))/2;


pz = (p_w(:,2:end) - p_w(:,1:end-1)) ./ dZ;

zpx = -(Z_r(2:end,:)-Z_r(1:end-1,:))/dx;


px1 = (p_r(2:end,:)-p_r(1:end-1,:))/dx;
px2 = (pz(1:end-1,:)+pz(2:end,:))/2  .* zpx;

px = px1 + px2;


if 0
  hold on
%  pcolor(X_r,Z_r,rho)
  pcolor(X_r,Z_r,p_r)
  colorbar
  plot(X_r,Z_r)
end

if 1
  subplot(2,2,1);
  hold on
  
  pcolor(X_w,Z_w,[rho NaN*ones(imax,1)])
  shading flat
  colorbar
  plot(X_w,Z_w,'k')
  title('Density');

  subplot(2,2,2);  
  pcolor(X_w(1:end-1,:),Z_w(1:end-1,:),[px NaN*ones(imax-1,1)])
  shading flat
  colorbar
  hold on
  plot(X_w(1:end-1,:),Z_w(1:end-1,:),'k')
  title('Pressure gradient');


  subplot(2,2,3);  
  pcolor(X_w(1:end-1,:),Z_w(1:end-1,:),[px1 NaN*ones(imax-1,1)])
  shading flat
  colorbar
  hold on
  plot(X_w(1:end-1,:),Z_w(1:end-1,:),'k')
  title('Pressure gradient along sigma');


  subplot(2,2,4);  
  pcolor(X_w(1:end-1,:),Z_w(1:end-1,:),[px2 NaN*ones(imax-1,1)])
  shading flat
  colorbar
  hold on
  plot(X_w(1:end-1,:),Z_w(1:end-1,:),'k')
  title('Pressure gradient due to tilting');
end

  set(gcf,'PaperUnits','centimeters','PaperPosition',[0 0 21 11])
print('-depsc','pressure_gradient.eps');