% parameters to define

if 1
  L=100e3;
  BH=2.6015e+10;
  AH=1;

  BH=1.6015e+10;
  AH=.5;

  epsilon = 0.03;
  
  dt = 1000;
  nstart = 0;
  nend = 240000;
  
  plot_every = -30; % if negative plots are disabled
  save_every = 100; % if negative output is disabled
end

more off

% number of grid points

M=124;
N=124;

% resolution

dx=20*L/M;
dy=dx;

dissrate=0.00000;
vorticity_scale=0.00001;
optsur=2*(1-pi/M);
bcflag=2;

% bcfalg=1: perdiodic along x
% bcflag=2: dirichlet with zero value everywhere
%
imode=2;

outdir = sprintf('~/tmp/gaussian_L%g_dt%g_AH%g_BH%g',L,dt,AH,BH/1e10);
disp(['saving results in ' outdir]);
mkdir(outdir);

% colorbar
cmap = bluered(64);

% 
for ic=1:M
  for jc=1:N
    vorticity(ic,jc)=0;
    xx(ic,jc)=(ic-M/2)*dx;
    yy(ic,jc)=(jc-N/2)*dy;
    rr=sqrt(xx(ic,jc)^2+yy(ic,jc)^2)/L;
    th=atan2(yy(ic,jc),xx(ic,jc));
    r=rr*(1+epsilon*cos(imode*th));
    %r=rr*(1+0.1*cos(imode*th));
    % 
    %streamfunc(ic,jc)=-(r+1)*exp(-r)*L^2*vorticity_scale;
    % Gaussian eddy
    streamfunc(ic,jc)=exp(-r*r)*L^2*vorticity_scale;
    % Gaussian jet
    %streamfunc(ic,jc)=exp(-(yy(ic,jc)+dx*sin(ic/(M-2)*2*pi))^2/L^2)*L^2*vorticity_scale;
    %bcflag=1;
  end
end


if bcflag==1
  streamfunc=periodic(streamfunc);
  vorticity=periodic(laplacian(streamfunc,dx,dy));
else
  streamfunc=dirichletbc(streamfunc,0);
  vorticity=dirichletbc(laplacian(streamfunc,dx,dy),0);
end

%pcolor(streamfunc')
%colormap(cmap)
%colorbar
%pause

if nstart ~= 0
  filename = sprintf('%s/shieldededdynew%05d.mat',outdir,nstart);
  disp(['loading IC ' filename]);
  load(filename);
  nstart = nstart+1;
end


for n=nstart:nend
  %for n=0:-1
  disp(['time step ' num2str(n)])
  
  % compute steam function from vorticity by inverting the poisson equation
  [streamfunc,nit,relerr,residual]=inversepoisson(vorticity,streamfunc,dx,dy,100,0.0000005,optsur);

  % compute J(streamfunc,vorticity), .i.e. advection of vorticity
  jacobphys=arakawa(streamfunc,vorticity,dx,dy);
  
  lap=laplacian(vorticity,dx,dy);

  % boundary condition
  if bcflag==1
    lap=periodic(lap);
  end

  % vorticity equation
  
  vorticity = vorticity*exp(-dissrate*dt) ...  % dissipation
      - dt*jacobphys                      ...  % advection
      + AH*dt*laplacian(vorticity,dx,dy)  ...  % harmonic diffusion
      - BH*dt*laplacian(lap,dx,dy);            % bi-harmonic diffusion

  % boundary condition
  if bcflag==1
    vorticity=periodic(vorticity);
  else
    vorticity=dirichletbc(vorticity,0);
  end

  %  sum(vorticity(:))
  %  vorticity(M/2,N/2)

  % plot results
  
  if mod(n,plot_every) == 0 && plot_every > 0
    clf
    pcolor(vorticity); shading flat; 
    title(['time step ' num2str(n)]); 
    axis equal
    colormap(cmap)
    caxis([-1 1]*vorticity_scale); 
    colorbar
    drawnow
  end
  
  % save results
  if mod(n,save_every) == 0 && save_every > 0
    filename = sprintf('%s/shieldededdynew%05d.mat',outdir,n);
    disp(['saving ' filename])
    save('-mat',filename,'streamfunc','vorticity');
  end
  
  % stop if error
  if any(isnan(vorticity(:)))
    error('NaN in vorticity');
  end  
end